@@ -29,9 +29,6 @@
 
 #include <time.h>
 #include <unistd.h>
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
-#include <malloc.h>
-#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/types.h>
@@ -175,7 +172,7 @@ XrdBuffer *XrdBuffManager::Obtain(int sz)
 // Allocate a chunk of aligned memory
 //
    pk = (mk < pagsz ? mk : pagsz);
-   if (!(memp = static_cast<char *>(memalign(pk, mk)))) return 0;
+   if (posix_memalign((void **)&memp, pk, mk)) return 0;
 
 // Wrap the memory with a buffer object
 //
