@@ -28,7 +28,6 @@
 #include "XrdCl/XrdClSocket.hh"
 #include "XrdCl/XrdClConstants.hh"
 #include "XrdCl/XrdClLog.hh"
-#include "XrdCl/XrdClUglyHacks.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
 #include "XrdCl/XrdClXRootDTransport.hh"
 
@@ -48,7 +47,7 @@ namespace
       // Constructor
       //------------------------------------------------------------------------
       FilterHandler( XrdCl::MessageFilter *filter ):
-        pSem( new XrdCl::Semaphore(0) ), pFilter( filter ), pMsg( 0 )
+        pSem( new XrdSysSemaphore(0) ), pFilter( filter ), pMsg( 0 )
       {
       }
 
@@ -70,6 +69,14 @@ namespace
         return Ignore;
       }
 
+      //------------------------------------------------------------------------
+      //! Reexamine the incoming message, and decide on the action to be taken
+      //------------------------------------------------------------------------
+      virtual uint16_t InspectStatusRsp( XrdCl::Message *msg )
+      {
+        return 0;
+      }
+
       virtual void Process( XrdCl::Message *msg )
       {
         pMsg = msg;
@@ -121,7 +128,7 @@ namespace
       FilterHandler(const FilterHandler &other);
       FilterHandler &operator = (const FilterHandler &other);
 
-      XrdCl::Semaphore     *pSem;
+      XrdSysSemaphore      *pSem;
       XrdCl::MessageFilter *pFilter;
       XrdCl::Message       *pMsg;
       XrdCl::XRootDStatus   pStatus;
@@ -137,7 +144,7 @@ namespace
       // Constructor
       //------------------------------------------------------------------------
       StatusHandler( XrdCl::Message *msg ):
-        pSem( new XrdCl::Semaphore(0) ),
+        pSem( new XrdSysSemaphore(0) ),
         pMsg( msg ) {}
 
       //------------------------------------------------------------------------
@@ -172,7 +179,7 @@ namespace
       StatusHandler(const StatusHandler &other);
       StatusHandler &operator = (const StatusHandler &other);
 
-      XrdCl::Semaphore    *pSem;
+      XrdSysSemaphore     *pSem;
       XrdCl::XRootDStatus  pStatus;
       XrdCl::Message      *pMsg;
   };
@@ -230,7 +237,8 @@ namespace XrdCl
                     Poller           *poller,
                     TransportHandler *transport,
                     TaskManager      *taskManager,
-                    JobManager       *jobManager ):
+                    JobManager       *jobManager,
+                    const URL        &prefurl ):
     pUrl( url.GetHostId() ),
     pPoller( poller ),
     pTransport( transport ),
@@ -254,14 +262,14 @@ namespace XrdCl
     //--------------------------------------------------------------------------
     // Create the stream
     //--------------------------------------------------------------------------
-      pStream = new Stream( &pUrl );
-      pStream->SetTransport( transport );
-      pStream->SetPoller( poller );
-      pStream->SetIncomingQueue( &pIncoming );
-      pStream->SetTaskManager( taskManager );
-      pStream->SetJobManager( jobManager );
-      pStream->SetChannelData( &pChannelData );
-      pStream->Initialize();
+    pStream = new Stream( &pUrl, prefurl );
+    pStream->SetTransport( transport );
+    pStream->SetPoller( poller );
+    pStream->SetIncomingQueue( &pIncoming );
+    pStream->SetTaskManager( taskManager );
+    pStream->SetJobManager( jobManager );
+    pStream->SetChannelData( &pChannelData );
+    pStream->Initialize();
 
     //--------------------------------------------------------------------------
     // Register the task generating timeout events
@@ -363,13 +371,29 @@ namespace XrdCl
   }
 
   //------------------------------------------------------------------------
-  //! Set the on-connect handler for data streams
+  // Set the on-connect handler for data streams
   //------------------------------------------------------------------------
   void Channel::SetOnDataConnectHandler( std::shared_ptr<Job> &onConnJob )
   {
     pStream->SetOnDataConnectHandler( onConnJob );
   }
 
+  //------------------------------------------------------------------------
+  // Check if channel can be collapsed using given URL
+  //------------------------------------------------------------------------
+  bool Channel::CanCollapse( const URL &url )
+  {
+    return pStream->CanCollapse( url );
+  }
+
+  //------------------------------------------------------------------------
+  // Decrement file object instance count bound to this channel
+  //------------------------------------------------------------------------
+  void Channel::DecFileInstCnt()
+  {
+    pTransport->DecFileInstCnt( pChannelData );
+  }
+
   //----------------------------------------------------------------------------
   // Query the transport handler
   //----------------------------------------------------------------------------
