@@ -42,7 +42,7 @@ namespace XrdCl
     pPlugIn(0),
     pEnablePlugIns( enablePlugIns )
   {
-    pStateHandler = new FileStateHandler();
+    pStateHandler = new FileStateHandler( pPlugIn );
   }
 
   //----------------------------------------------------------------------------
@@ -52,7 +52,7 @@ namespace XrdCl
     pPlugIn(0),
     pEnablePlugIns( enablePlugIns )
   {
-    pStateHandler = new FileStateHandler( virtRedirect == EnableVirtRedirect );
+    pStateHandler = new FileStateHandler( virtRedirect == EnableVirtRedirect, pPlugIn );
   }
 
   //------------------------------------------------------------------------
@@ -144,7 +144,7 @@ namespace XrdCl
   {
     SyncResponseHandler handler;
     XRootDStatus st = Close( &handler, timeout );
-    if( !st.IsOK() )
+    if( !st.IsOK() || st.code == suAlreadyDone )
       return st;
 
     return MessageUtils::WaitForStatus( &handler );
@@ -227,7 +227,10 @@ namespace XrdCl
                              ResponseHandler *handler,
                              uint16_t         timeout )
   {
-    return XRootDStatus( stError, errNotImplemented );
+    if( pPlugIn )
+      return pPlugIn->PgRead( offset, size, buffer, handler, timeout );
+
+    return pStateHandler->PgRead( offset, size, buffer, handler, timeout );
   }
 
   //------------------------------------------------------------------------
@@ -236,11 +239,24 @@ namespace XrdCl
   XRootDStatus File::PgRead( uint64_t               offset,
                              uint32_t               size,
                              void                  *buffer,
-                             uint32_t              &bytesRead,
                              std::vector<uint32_t> &cksums,
+                             uint32_t              &bytesRead,
                              uint16_t               timeout )
   {
-    return XRootDStatus( stError, errNotImplemented );
+    SyncResponseHandler handler;
+    XRootDStatus st = PgRead( offset, size, buffer, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    PageInfo *pageInfo = 0;
+    XRootDStatus status = MessageUtils::WaitForResponse( &handler, pageInfo );
+    if( status.IsOK() )
+    {
+      bytesRead = pageInfo->GetLength();
+      cksums = pageInfo->GetCksums();
+      delete pageInfo;
+    }
+    return status;
   }
 
   //----------------------------------------------------------------------------
@@ -275,6 +291,68 @@ namespace XrdCl
     return status;
   }
 
+
+  XRootDStatus File::Write( uint64_t          offset,
+                            Buffer          &&buffer,
+                            ResponseHandler  *handler,
+                            uint16_t          timeout )
+  {
+    if( pPlugIn )
+      return pPlugIn->Write( offset, std::move( buffer ), handler, timeout );
+
+    return pStateHandler->Write( offset, std::move( buffer ), handler, timeout );
+  }
+
+  //----------------------------------------------------------------------------
+  // Write a data chunk at a given offset - async
+  //----------------------------------------------------------------------------
+  XRootDStatus File::Write( uint64_t    offset,
+                            Buffer    &&buffer,
+                            uint16_t    timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = Write( offset, std::move( buffer ), &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    XRootDStatus status = MessageUtils::WaitForStatus( &handler );
+    return status;
+  }
+
+  //------------------------------------------------------------------------
+  // Write a data from a given file descriptor at a given offset - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::Write( uint64_t            offset,
+                            uint32_t            size,
+                            Optional<uint64_t>  fdoff,
+                            int                 fd,
+                            ResponseHandler    *handler,
+                            uint16_t            timeout )
+  {
+    if( pPlugIn )
+      return pPlugIn->Write( offset, size, fdoff, fd, handler, timeout );
+
+    return pStateHandler->Write( offset, size, fdoff, fd, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Write a data from a given file descriptor at a given offset - sync
+  //------------------------------------------------------------------------
+  XRootDStatus File::Write( uint64_t            offset,
+                            uint32_t            size,
+                            Optional<uint64_t>  fdoff,
+                            int                 fd,
+                            uint16_t            timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = Write( offset, size, fdoff, fd, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    XRootDStatus status = MessageUtils::WaitForStatus( &handler );
+    return status;
+  }
+
   //------------------------------------------------------------------------
   // Write number of pages at a given offset - async
   //------------------------------------------------------------------------
@@ -392,7 +470,7 @@ namespace XrdCl
                             uint16_t         timeout )
   {
     if( pPlugIn )
-      return XRootDStatus( stError, errNotSupported );
+      return pPlugIn->VectorWrite( chunks, handler, timeout );
 
     return pStateHandler->VectorWrite( chunks, handler, timeout );
   }
@@ -415,12 +493,13 @@ namespace XrdCl
   // Write scattered buffers in one operation - async
   //------------------------------------------------------------------------
   XRootDStatus File::WriteV( uint64_t            offset,
-                                  const struct iovec *iov,
-                                  int                 iovcnt,
-                                  ResponseHandler    *handler,
-                                  uint16_t            timeout )
+                             const struct iovec *iov,
+                             int                 iovcnt,
+                             ResponseHandler    *handler,
+                             uint16_t            timeout )
   {
-    // TODO check pPlugIn
+    if( pPlugIn )
+      return pPlugIn->WriteV( offset, iov, iovcnt, handler, timeout );
 
     return pStateHandler->WriteV( offset, iov, iovcnt, handler, timeout );
   }
@@ -429,9 +508,9 @@ namespace XrdCl
   // Write scattered buffers in one operation - sync
   //------------------------------------------------------------------------
   XRootDStatus File::WriteV( uint64_t            offset,
-                                  const struct iovec *iov,
-                                  int                 iovcnt,
-                                  uint16_t            timeout )
+                             const struct iovec *iov,
+                             int                 iovcnt,
+                             uint16_t            timeout )
   {
     SyncResponseHandler handler;
     XRootDStatus st = WriteV( offset, iov, iovcnt, &handler, timeout );
@@ -442,6 +521,57 @@ namespace XrdCl
     return status;
   }
 
+  //------------------------------------------------------------------------
+  //! Read data into scattered buffers in one operation - async
+  //!
+  //! @param offset    offset from the beginning of the file
+  //! @param iov       list of the buffers to be written
+  //! @param iovcnt    number of buffers
+  //! @param handler   handler to be notified when the response arrives
+  //! @param timeout   timeout value, if 0 then the environment default
+  //!                  will be used
+  //! @return          status of the operation
+  //------------------------------------------------------------------------
+  XRootDStatus File::ReadV( uint64_t         offset,
+                            struct iovec    *iov,
+                            int              iovcnt,
+                            ResponseHandler *handler,
+                            uint16_t         timeout )
+  {
+    return pStateHandler->ReadV( offset, iov, iovcnt, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  //! Read data into scattered buffers in one operation - sync
+  //!
+  //! @param offset    offset from the beginning of the file
+  //! @param iov       list of the buffers to be written
+  //! @param iovcnt    number of buffers
+  //! @param handler   handler to be notified when the response arrives
+  //! @param timeout   timeout value, if 0 then the environment default
+  //!                  will be used
+  //! @return          status of the operation
+  //------------------------------------------------------------------------
+  XRootDStatus File::ReadV( uint64_t      offset,
+                            struct iovec *iov,
+                            int           iovcnt,
+                            uint32_t     &bytesRead,
+                            uint16_t      timeout )
+  {
+    SyncResponseHandler handler;
+    XRootDStatus st = ReadV( offset, iov, iovcnt, &handler, timeout );
+    if( !st.IsOK() )
+      return st;
+
+    VectorReadInfo *vrInfo = 0;
+    XRootDStatus status = MessageUtils::WaitForResponse( &handler, vrInfo );
+    if( status.IsOK() )
+    {
+      bytesRead = vrInfo->GetSize();
+      delete vrInfo;
+    }
+    return status;
+  }
 
   //----------------------------------------------------------------------------
   // Performs a custom operation on an open file, server implementation
@@ -629,6 +759,56 @@ namespace XrdCl
     return st;
   }
 
+  //------------------------------------------------------------------------
+  // Create a checkpoint
+  //------------------------------------------------------------------------
+  XRootDStatus File::Checkpoint( kXR_char                  code,
+                                 ResponseHandler          *handler,
+                                 uint16_t                  timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->Checkpoint( code, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  //! Checkpointed write - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::ChkptWrt( uint64_t         offset,
+                               uint32_t         size,
+                               const void      *buffer,
+                               ResponseHandler *handler,
+                               uint16_t         timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->ChkptWrt( offset, size, buffer, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  //! Checkpointed WriteV - async
+  //------------------------------------------------------------------------
+  XRootDStatus File::ChkptWrtV( uint64_t            offset,
+                                const struct iovec *iov,
+                                int                 iovcnt,
+                                ResponseHandler    *handler,
+                                uint16_t            timeout )
+  {
+    if( pPlugIn )
+      return XRootDStatus( stError, errNotSupported );
+
+    return pStateHandler->ChkptWrtV( offset, iov, iovcnt, handler, timeout );
+  }
+
+  //------------------------------------------------------------------------
+  // Try different data server
+  //------------------------------------------------------------------------
+  XRootDStatus File::TryOtherServer( uint16_t timeout )
+  {
+    return pStateHandler->TryOtherServer( timeout );
+  }
 
   //----------------------------------------------------------------------------
   // Check if the file is open
