@@ -945,14 +945,26 @@ namespace XrdCl
       log->Dump( FileSystemMsg, "[0x%x@%s] Sending %s", this,
                  pUrl->GetHostId().c_str(), msg->GetDescription().c_str() );
 
-      handler = new AssignLastURLHandler( this, handler );
+      AssignLastURLHandler *lastUrlHandler = new AssignLastURLHandler( this, handler );
+      handler = lastUrlHandler;
 
+      AssignLBHandler *lbHandler = nullptr;
       if( !pLoadBalancerLookupDone && pFollowRedirects )
-        handler = new AssignLBHandler( this, handler ); // TODO
+      {
+        lbHandler = new AssignLBHandler( this, handler );
+        handler = lbHandler;
+      }
 
       params.followRedirects = pFollowRedirects;
 
-      return MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
+      auto st = MessageUtils::SendMessage( *pUrl, msg, handler, params, 0 );
+      if( !st.IsOK() )
+      {
+        delete lastUrlHandler;
+        delete lbHandler;
+      }
+
+      return st;
     }
 
     //----------------------------------------------------------------------------
@@ -1014,25 +1026,41 @@ namespace XrdCl
     }
 
     bool finalrsp = !( status->IsOK() && status->code == suContinue );
-    pUserHandler->HandleResponseWithHosts( status, response, hostList );
+
+    SyncResponseHandler * syncHandler = dynamic_cast<SyncResponseHandler*>( pUserHandler );
+    if( !syncHandler )
+      pUserHandler->HandleResponseWithHosts( status, response, hostList );
+
     if( finalrsp )
+    {
+      if( syncHandler )
+        pUserHandler->HandleResponseWithHosts( status, response, hostList );
       delete this;
+    }
   }
 
   //------------------------------------------------------------------------
   // Response callback
   //------------------------------------------------------------------------
   void AssignLastURLHandler::HandleResponseWithHosts( XRootDStatus *status,
-                                               AnyObject    *response,
-                                               HostList     *hostList )
+                                                      AnyObject    *response,
+                                                      HostList     *hostList )
   {
     if( status->IsOK() && hostList )
       pFS->AssignLastURL( hostList->front().url );
 
     bool finalrsp = !( status->IsOK() && status->code == suContinue );
-    pUserHandler->HandleResponseWithHosts( status, response, hostList );
+
+    SyncResponseHandler *syncHandler = dynamic_cast<SyncResponseHandler*>( pUserHandler );
+    if( !syncHandler )
+      pUserHandler->HandleResponseWithHosts( status, response, hostList );
+
     if( finalrsp )
+    {
+      if( syncHandler )
+        pUserHandler->HandleResponseWithHosts( status, response, hostList );
       delete this;
+    }
   }
 
   //----------------------------------------------------------------------------
@@ -1651,6 +1679,9 @@ namespace XrdCl
     if( ( flags & DirListFlags::Stat ) || ( flags & DirListFlags::Recursive ) )
       req->options[0] = kXR_dstat;
 
+    if( ( flags & DirListFlags::Cksm ) )
+      req->options[0] = kXR_dstat | kXR_dcksm;
+
     if( flags & DirListFlags::Recursive )
       handler = new RecursiveDirListHandler( *pImpl->pUrl, url.GetPath(), flags, handler, timeout );
 
