@@ -29,10 +29,10 @@
 #include "XrdCl/XrdClUtils.hh"
 #include "XrdCl/XrdClMessageUtils.hh"
 #include "XrdCl/XrdClMonitor.hh"
-#include "XrdCl/XrdClUglyHacks.hh"
 #include "XrdCl/XrdClRedirectorRegistry.hh"
 #include "XrdCl/XrdClDlgEnv.hh"
 #include "XrdOuc/XrdOucTPC.hh"
+#include "XrdSys/XrdSysPthread.hh"
 #include "XrdSys/XrdSysTimer.hh"
 
 #include <iostream>
@@ -58,7 +58,7 @@ namespace
       // Constructor
       //------------------------------------------------------------------------
       TPCStatusHandler():
-        pSem( new XrdCl::Semaphore(0) ), pStatus(0)
+        pSem( new XrdSysSemaphore(0) ), pStatus(0)
       {
       }
 
@@ -85,7 +85,7 @@ namespace
       //------------------------------------------------------------------------
       // Get Mutex
       //------------------------------------------------------------------------
-      XrdCl::Semaphore *GetSemaphore()
+      XrdSysSemaphore *GetXrdSysSemaphore()
       {
         return pSem;
       }
@@ -102,7 +102,7 @@ namespace
       TPCStatusHandler(const TPCStatusHandler &other);
       TPCStatusHandler &operator = (const TPCStatusHandler &other);
 
-      XrdCl::Semaphore    *pSem;
+      XrdSysSemaphore     *pSem;
       XrdCl::XRootDStatus *pStatus;
   };
 
@@ -219,7 +219,8 @@ namespace XrdCl
         if( !checkSumPreset.empty() )
         {
           sourceCheckSum  = checkSumType + ":";
-          sourceCheckSum += checkSumPreset;
+          sourceCheckSum += Utils::NormalizeChecksum( checkSumType,
+                                                      checkSumPreset );
         }
         else
         {
@@ -230,9 +231,7 @@ namespace XrdCl
               !( vrCheckSum = redirector->GetCheckSum( checkSumType ) ).empty() )
             sourceCheckSum = vrCheckSum;
           else
-            st = Utils::GetRemoteCheckSum( sourceCheckSum, checkSumType,
-                                           tpcSource.GetHostId(),
-                                           tpcSource.GetPath() );
+            st = Utils::GetRemoteCheckSum( sourceCheckSum, checkSumType, tpcSource );
         }
         gettimeofday( &oEnd, 0 );
         if( !st.IsOK() )
@@ -249,9 +248,7 @@ namespace XrdCl
       if( checkSumMode == "end2end" || checkSumMode == "target" )
       {
         gettimeofday( &tStart, 0 );
-        st = Utils::GetRemoteCheckSum( targetCheckSum, checkSumType,
-                                       realTarget.GetHostId(),
-                                       realTarget.GetPath() );
+        st = Utils::GetRemoteCheckSum( targetCheckSum, checkSumType, realTarget );
 
         gettimeofday( &tEnd, 0 );
         if( !st.IsOK() )
@@ -259,6 +256,22 @@ namespace XrdCl
         pResults->Set( "targetCheckSum", targetCheckSum );
       }
 
+      //------------------------------------------------------------------------
+      // Make sure the checksums are both lower case
+      //------------------------------------------------------------------------
+      auto sanitize_cksum = []( char c )
+                            {
+                              std::locale loc;
+                              if( std::isalpha( c ) ) return std::tolower( c, loc );
+                              return c;
+                            };
+
+      std::transform( sourceCheckSum.begin(), sourceCheckSum.end(),
+                      sourceCheckSum.begin(), sanitize_cksum );
+
+      std::transform( targetCheckSum.begin(), targetCheckSum.end(),
+                      targetCheckSum.begin(), sanitize_cksum );
+
       //------------------------------------------------------------------------
       // Compare and inform monitoring
       //------------------------------------------------------------------------
@@ -373,7 +386,8 @@ namespace XrdCl
     // Do the facultative step at source only if the protocol is root/xroot,
     // otherwise don't bother
     //--------------------------------------------------------------------------
-    if( sourceURL.GetProtocol() == "root" || sourceURL.GetProtocol() == "xroot" )
+    if( sourceURL.GetProtocol() == "root"  || sourceURL.GetProtocol() == "xroot" ||
+        sourceURL.GetProtocol() == "roots" || sourceURL.GetProtocol() == "xroots" )
     {
       params = sourceURL.GetParams();
       params["tpc.stage"] = "placement";
@@ -495,8 +509,11 @@ namespace XrdCl
     params = realTarget.GetParams();
     MessageUtils::MergeCGI( params, cgiURL.GetParams(), true );
 
-    std::ostringstream o; o << sourceSize;
-    params["oss.asize"] = o.str();
+    if( !tpcLiteOnly ) // we only append oss.asize if it source file size is actually known
+    {
+      std::ostringstream o; o << sourceSize;
+      params["oss.asize"] = o.str();
+    }
     params["tpc.stage"] = "copy";
 
     // forward source cgi info to the destination in case we are going to do delegation
@@ -685,7 +702,7 @@ namespace XrdCl
     // Do the copy and follow progress
     //--------------------------------------------------------------------------
     TPCStatusHandler  statusHandler;
-    Semaphore        *sem  = statusHandler.GetSemaphore();
+    XrdSysSemaphore  *sem  = statusHandler.GetXrdSysSemaphore();
     StatInfo         *info   = 0;
 
     uint16_t tpcTimeout = 0;
@@ -707,7 +724,7 @@ namespace XrdCl
     bool canceled = false;
     while( 1 )
     {
-      XrdSysTimer::Wait( 1000 );
+      XrdSysTimer::Wait( 2500 );
 
       if( progress )
       {
@@ -804,7 +821,7 @@ namespace XrdCl
     // Do the copy and follow progress
     //--------------------------------------------------------------------------
     TPCStatusHandler  statusHandler;
-    Semaphore        *sem  = statusHandler.GetSemaphore();
+    XrdSysSemaphore  *sem  = statusHandler.GetXrdSysSemaphore();
     StatInfo         *info   = 0;
 
     uint16_t tpcTimeout = 0;
@@ -825,7 +842,7 @@ namespace XrdCl
     bool canceled = false;
     while( 1 )
     {
-      XrdSysTimer::Wait( 1000 );
+      XrdSysTimer::Wait( 2500 );
 
       if( progress )
       {
