@@ -39,6 +39,7 @@
 
 #include "XrdOuc/XrdOucName2Name.hh"
 #include "XrdPosix/XrdPosixCallBack.hh"
+#include "XrdPosix/XrdPosixConfig.hh"
 #include "XrdPosix/XrdPosixFile.hh"
 #include "XrdPosix/XrdPosixFileRH.hh"
 #include "XrdPosix/XrdPosixPrepIO.hh"
@@ -47,6 +48,7 @@
 #include "XrdPosix/XrdPosixXrootdPath.hh"
 
 #include "XrdSys/XrdSysError.hh"
+#include "XrdSys/XrdSysPageSize.hh"
 #include "XrdSys/XrdSysTimer.hh"
 
 /******************************************************************************/
@@ -61,6 +63,7 @@ extern XrdSysError     *eDest;
 extern XrdPosixStats    Stats;
 extern int              ddInterval;
 extern int              ddMaxTries;
+extern bool             autoPGRD;
 };
 
 namespace
@@ -85,6 +88,31 @@ short          XrdPosixFile::sfSLN    =  0;
 bool           XrdPosixFile::ddPosted = false;
 int            XrdPosixFile::ddNum    =  0;
 
+/******************************************************************************/
+/*                         L o c a l   C l a s s e s                          */
+/******************************************************************************/
+  
+namespace
+{
+class pgioCB : public XrdOucCacheIOCB
+{
+public:
+
+void Done(int result)
+         {rc = result; pgSem.Post();}
+
+int  Wait4PGIO()      {pgSem.Wait(); return rc;}
+
+     pgioCB(const char *who) : pgSem(0, who), rc(0) {}
+    ~pgioCB() {}
+
+private:
+
+XrdSysSemaphore pgSem;
+int             rc;
+};
+}
+
 /******************************************************************************/
 /*                           C o n s t r u c t o r                            */
 /******************************************************************************/
@@ -92,8 +120,8 @@ int            XrdPosixFile::ddNum    =  0;
 XrdPosixFile::XrdPosixFile(bool &aOK, const char *path, XrdPosixCallBack *cbP,
                            int Opts)
              : XCio((XrdOucCacheIO *)this), PrepIO(0),
-               mySize(0), myMtime(0), myInode(0), myMode(0),
-               theCB(cbP), fLoc(0), cOpt(0),
+               mySize(0), myAtime(0), myCtime(0), myMtime(0), myRdev(0),
+               myInode(0), myMode(0), theCB(cbP), fLoc(0), cOpt(0),
                isStream(Opts & isStrm ? 1 : 0)
 {
 // Handle path generation. This is trickt as we may have two namespaces. One
@@ -345,9 +373,12 @@ int XrdPosixFile::Fstat(struct stat &buf)
 
 // Return what little we can
 //
+   XrdPosixConfig::initStat(&buf);
    buf.st_size   = theSize;
-   buf.st_atime  = buf.st_mtime = buf.st_ctime = myMtime;
-   buf.st_blocks = buf.st_size/512+1;
+   buf.st_atime  = myAtime;
+   buf.st_ctime  = myCtime;
+   buf.st_mtime  = myMtime;
+   buf.st_blocks = buf.st_size/512 + buf.st_size%512;
    buf.st_ino    = myInode;
    buf.st_rdev   = myRdev;
    buf.st_mode   = myMode;
@@ -388,7 +419,7 @@ void XrdPosixFile::HandleResponse(XrdCl::XRootDStatus *status,
 /*                              L o c a t i o n                               */
 /******************************************************************************/
   
-const char *XrdPosixFile::Location()
+const char *XrdPosixFile::Location(bool refresh)
 {
 
 // If the file is not open, then we have no location
@@ -397,10 +428,14 @@ const char *XrdPosixFile::Location()
 
 // If we have no location info, get it
 //
-   if (!fLoc)
+   if (!fLoc || refresh)
       {std::string currNode;
        if (clFile.GetProperty(dsProperty, currNode))
-          fLoc = strdup(currNode.c_str());
+          {if (!fLoc || strcmp(fLoc, currNode.c_str()))
+              {if (fLoc) free(fLoc);
+               fLoc = strdup(currNode.c_str());
+              }
+          } else return "";
       }
 
 // Return location information
@@ -408,6 +443,108 @@ const char *XrdPosixFile::Location()
    return fLoc;
 }
 
+/******************************************************************************/
+/*                                p g R e a d                                 */
+/******************************************************************************/
+
+int XrdPosixFile::pgRead(char                  *buff,
+                         long long              offs,
+                         int                    rlen,
+                         std::vector<uint32_t> &csvec,
+                         uint64_t               opts)
+{
+// Do a sync call using the async interface
+//
+   pgioCB pgrCB("Posix pgRead CB");
+   pgRead(pgrCB, buff, offs, rlen, csvec, opts);
+   return pgrCB.Wait4PGIO();
+}
+  
+/******************************************************************************/
+
+void XrdPosixFile::pgRead(XrdOucCacheIOCB       &iocb,
+                          char                  *buff,
+                          long long              offs,
+                          int                    rlen,
+                          std::vector<uint32_t> &csvec,
+                          uint64_t               opts)
+{
+   XrdCl::XRootDStatus Status;
+   XrdPosixFileRH *rhP;
+
+// Allocate callback object. Note the response handler may do additional post
+// processing.
+//
+   rhP = XrdPosixFileRH::Alloc(&iocb, this, offs, rlen, XrdPosixFileRH::isReadP);
+
+// Set the destination checksum vector
+//
+   rhP->setCSVec(&csvec, (opts & XrdOucCacheIO::forceCS) != 0);
+
+// Issue read
+//
+   Ref();
+   Status = clFile.PgRead((uint64_t)offs,(uint32_t)rlen,buff,rhP);
+
+// Check status
+//
+   if (!Status.IsOK())
+      {rhP->Sched(XrdPosixMap::Result(Status, false));
+       unRef();
+      }
+}
+
+/******************************************************************************/
+/*                               p g W r i t e                                */
+/******************************************************************************/
+
+int XrdPosixFile::pgWrite(char                  *buff,
+                          long long              offs,
+                          int                    wlen,
+                          std::vector<uint32_t> &csvec,
+                          uint64_t               opts)
+{
+   XrdCl::XRootDStatus Status;
+
+// Issue write and return appropriately
+//
+   Ref();
+   Status = clFile.PgWrite((uint64_t)offs, (uint32_t)wlen, buff, csvec);
+   unRef();
+
+   return (Status.IsOK() ? wlen : XrdPosixMap::Result(Status));
+}
+  
+/******************************************************************************/
+
+void XrdPosixFile::pgWrite(XrdOucCacheIOCB       &iocb,
+                           char                  *buff,
+                           long long              offs,
+                           int                    wlen,
+                           std::vector<uint32_t> &csvec,
+                           uint64_t               opts)
+{
+   XrdCl::XRootDStatus Status;
+   XrdPosixFileRH *rhP;
+
+// Allocate callback object. Note that a pgWrite is essentially a normal write
+// as far as the response handler is concerned.
+//
+   rhP = XrdPosixFileRH::Alloc(&iocb,this,offs,wlen,XrdPosixFileRH::isWrite);
+
+// Issue write
+//
+   Ref();
+   Status = clFile.PgWrite((uint64_t)offs, (uint32_t)wlen, buff, csvec, rhP);
+
+// Check status
+//
+   if (!Status.IsOK())
+      {rhP->Sched(XrdPosixMap::Result(Status));
+       unRef();
+      }
+}
+
 /******************************************************************************/
 /*                                  R e a d                                   */
 /******************************************************************************/
@@ -417,6 +554,14 @@ int XrdPosixFile::Read (char *Buff, long long Offs, int Len)
    XrdCl::XRootDStatus Status;
    uint32_t bytes;
 
+// Handle automatic pgread
+//
+   if (XrdPosixGlobals::autoPGRD)
+      {pgioCB pgrCB("Posix pgRead CB");
+       Read(pgrCB, Buff, Offs, Len);
+       return pgrCB.Wait4PGIO();
+      }
+
 // Issue read and return appropriately.
 //
    Ref();
@@ -432,18 +577,25 @@ void XrdPosixFile::Read (XrdOucCacheIOCB &iocb, char *buff, long long offs,
                          int rlen)
 {
    XrdCl::XRootDStatus Status;
-   XrdPosixFileRH *rhp =  XrdPosixFileRH::Alloc(&iocb, this, offs, rlen,
-                                                XrdPosixFileRH::isRead);
+   XrdPosixFileRH *rhP;
+   XrdPosixFileRH::ioType rhT;
+   bool doPgRd = XrdPosixGlobals::autoPGRD;
+
+// Allocate correct callback object
+//
+   rhT = (doPgRd ? XrdPosixFileRH::isReadP : XrdPosixFileRH::isRead);
+   rhP = XrdPosixFileRH::Alloc(&iocb, this, offs, rlen, rhT);
 
 // Issue read
 //
    Ref();
-   Status = clFile.Read((uint64_t)offs, (uint32_t)rlen, buff, rhp);
+   if (doPgRd) Status = clFile.PgRead((uint64_t)offs,(uint32_t)rlen,buff,rhP);
+      else     Status = clFile.Read  ((uint64_t)offs,(uint32_t)rlen,buff,rhP);
 
 // Check status
 //
    if (!Status.IsOK())
-      {rhp->Sched(XrdPosixMap::Result(Status, false));
+      {rhP->Sched(XrdPosixMap::Result(Status, false));
        unRef();
       }
 }
@@ -535,13 +687,24 @@ bool XrdPosixFile::Stat(XrdCl::XRootDStatus &Status, bool force)
        return false;
       }
 
-// Copy over the relevant fields
+// Copy over the relevant fields, the stat structure must have been
+// properly pre-initialized.
 //
    myMode  = XrdPosixMap::Flags2Mode(&myRdev, sInfo->GetFlags());
    myMtime = static_cast<time_t>(sInfo->GetModTime());
    mySize  = static_cast<size_t>(sInfo->GetSize());
    myInode = static_cast<ino_t>(strtoll(sInfo->GetId().c_str(), 0, 10));
 
+// If this is an extended stat then we can get some more info
+//
+   if (sInfo->ExtendedFormat())
+      {myCtime = static_cast<time_t>(sInfo->GetChangeTime());
+       myAtime = static_cast<time_t>(sInfo->GetAccessTime());
+      } else {
+       myCtime = myMtime;
+       myAtime = time(0);
+      }
+
 // Delete our status information and return final result
 //
    unRef();
@@ -612,7 +775,7 @@ int XrdPosixFile::Write(char *Buff, long long Offs, int Len)
 {
    XrdCl::XRootDStatus Status;
 
-// Issue read and return appropriately
+// Issue write and return appropriately
 //
    Ref();
    Status = clFile.Write((uint64_t)Offs, (uint32_t)Len, Buff);
@@ -630,7 +793,7 @@ void XrdPosixFile::Write(XrdOucCacheIOCB &iocb, char *buff, long long offs,
    XrdPosixFileRH *rhp =  XrdPosixFileRH::Alloc(&iocb, this, offs, wlen,
                                                 XrdPosixFileRH::isWrite);
 
-// Issue read
+// Issue write
 //
    Ref();
    Status = clFile.Write((uint64_t)offs, (uint32_t)wlen, buff, rhp);
