@@ -93,6 +93,7 @@ XrdCl::DirListFlags::Flags dlFlag = XrdCl::DirListFlags::None;
 bool             oidsOK    = false;
 bool             p2lSRC    = false;
 bool             p2lSGI    = false;
+bool             autoPGRD  = false;
 };
 
 int            XrdPosixXrootd::baseFD    = 0;
@@ -187,13 +188,18 @@ XrdPosixXrootd::XrdPosixXrootd(int fdnum, int dirnum, int thrnum)
    myMutex.UnLock();
 
 // Initialize environment as a client or a server (it differs somewhat).
+// Note that we create a permanent Env since some plugins rely on it. We
+// leave the logger handling to OucPsx as we do not want to enable messages
+// because this is likely a client application that doesn't understand noise.
 //
    if (!XrdPosixGlobals::theLogger && (cfn=getenv("XRDPOSIX_CONFIG")) && *cfn)
       {bool hush;
        if (*cfn == '+') {hush = false; cfn++;}
           else hush = (getenv("XRDPOSIX_DEBUG") == 0);
        if (*cfn)
-          {XrdOucPsx psxConfig(&XrdVERSIONINFOVAR(XrdPosix), cfn);
+          {XrdOucEnv *psxEnv = new XrdOucEnv;
+           psxEnv->Put("psx.Client", "1");
+           XrdOucPsx psxConfig(&XrdVERSIONINFOVAR(XrdPosix), cfn, 0, psxEnv);
            if (!psxConfig.ClientConfig("posix.", hush)
            ||  !XrdPosixConfig::SetConfig(psxConfig))
               {std::cerr <<"Posix: Unable to instantiate specified "
@@ -364,7 +370,7 @@ int     XrdPosixXrootd::Fstat(int fildes, struct stat *buf)
 
 // First initialize the stat buffer
 //
-   initStat(buf);
+   XrdPosixConfig::initStat(buf);
 
 // Check if we can get the stat information from the cache.
 //
@@ -948,7 +954,7 @@ struct dirent* XrdPosixXrootd::Readdir(DIR *dirp)
    dp32 = (struct dirent *)dp64;
    if (dp32->d_name  != dp64->d_name)
       {dp32->d_ino    = dp64->d_ino;
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__GNU__) && !(defined(__FreeBSD_kernel__) && defined(__GLIBC__))
        dp32->d_off     = dp64->d_off;
 #endif
 #ifndef __solaris__
@@ -997,7 +1003,7 @@ int XrdPosixXrootd::Readdir_r(DIR *dirp,   struct dirent    *entry,
       {*result = 0; return rc;}
 
    entry->d_ino    = dp64->d_ino;
-#if !defined(__APPLE__) && !defined(__FreeBSD__)
+#if !defined(__APPLE__) && !defined(__FreeBSD__) && !defined(__GNU__) && !(defined(__FreeBSD_kernel__) && defined(__GLIBC__))
    entry->d_off    = dp64->d_off;
 #endif
 #ifndef __solaris__
@@ -1130,11 +1136,6 @@ void XrdPosixXrootd::Seekdir(DIR *dirp, long loc)
 int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 {
    XrdPosixAdmin admin(path);
-   size_t stSize;
-   dev_t  stRdev;
-   ino_t  stId;
-   time_t stMtime;
-   mode_t stFlags;
 
 // Make sure the admin is OK
 //
@@ -1142,7 +1143,7 @@ int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 
 // Initialize the stat buffer
 //
-   initStat(buf);
+   XrdPosixConfig::initStat(buf);
 
 // Check if we can get the stat informatation from the cache
 //
@@ -1156,16 +1157,7 @@ int XrdPosixXrootd::Stat(const char *path, struct stat *buf)
 
 // Issue the stat and verify that all went well
 //
-   if (!admin.Stat(&stFlags, &stMtime, &stSize, &stId, &stRdev)) return -1;
-
-// Return what little we can
-//
-   buf->st_size   = stSize;
-   buf->st_blocks = stSize/512+1;
-   buf->st_atime  = buf->st_mtime = buf->st_ctime = stMtime;
-   buf->st_ino    = stId;
-   buf->st_rdev   = stRdev;
-   buf->st_mode   = stFlags;
+   if (!admin.Stat(*buf)) return -1;
    return 0;
 }
 
@@ -1190,17 +1182,17 @@ int XrdPosixXrootd::Statfs(const char *path, struct statfs *buf)
    buf->f_bfree   = myVfs.f_bfree;
    buf->f_files   = myVfs.f_files;
    buf->f_ffree   = myVfs.f_ffree;
-#if defined(__APPLE__) || defined(__FreeBSD__)
+#if defined(__APPLE__) || defined(__FreeBSD__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
    buf->f_iosize  = myVfs.f_frsize;
 #else
    buf->f_frsize  = myVfs.f_frsize;
 #endif
-#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__)
+#if defined(__linux__) || defined(__APPLE__) || defined(__FreeBSD__) || defined(__GNU__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
    buf->f_bavail  = myVfs.f_bavail;
 #endif
-#if defined(__linux__)
+#if defined(__linux__) || defined(__GNU__)
    buf->f_namelen = myVfs.f_namemax;
-#elif defined(__FreeBSD__)
+#elif defined(__FreeBSD__) || (defined(__FreeBSD_kernel__) && defined(__GLIBC__))
    buf->f_namemax = myVfs.f_namemax;
 #endif
    return 0;
@@ -1472,51 +1464,3 @@ int XrdPosixXrootd::Fault(XrdPosixFile *fp, int ecode)
    errno = ecode;
    return -1;
 }
-
-/******************************************************************************/
-/*                              i n i t S t a t                               */
-/******************************************************************************/
-
-void XrdPosixXrootd::initStat(struct stat *buf)
-{
-   static int initStat = 0;
-   static dev_t st_rdev;
-   static dev_t st_dev;
-   static uid_t myUID = getuid();
-   static gid_t myGID = getgid();
-
-// Initialize the xdev fields. This cannot be done in the constructor because
-// we may not yet have resolved the C-library symbols.
-//
-   if (!initStat) {initStat = 1; initXdev(st_dev, st_rdev);}
-   memset(buf, 0, sizeof(struct stat));
-
-// Preset common fields
-//
-   buf->st_blksize= 64*1024;
-   buf->st_dev    = st_dev;
-   buf->st_rdev   = st_rdev;
-   buf->st_nlink  = 1;
-   buf->st_uid    = myUID;
-   buf->st_gid    = myGID;
-}
-  
-/******************************************************************************/
-/*                              i n i t X d e v                               */
-/******************************************************************************/
-  
-void XrdPosixXrootd::initXdev(dev_t &st_dev, dev_t &st_rdev)
-{
-   static dev_t tDev, trDev;
-   static bool aOK = false;
-   struct stat buf;
-
-// Get the device id for /tmp used by stat()
-//
-   if (aOK) {st_dev = tDev; st_rdev = trDev;}
-      else if (stat("/tmp", &buf)) {st_dev = 0; st_rdev = 0;}
-              else {st_dev  = tDev  = buf.st_dev;
-                    st_rdev = trDev = buf.st_rdev;
-                    aOK = true;
-                   }
-}
