@@ -133,7 +133,7 @@ do{ctxImpl->crlMutex.ReadLock();
 
 // Verify that the context was properly built
 //
-   if (!newctx->isOK())
+   if (!newctx || !newctx->isOK())
       {XrdTlsGlobal::msgCB("CrlRefresh:","Refresh of context failed!!!",false);
        continue;
       }
@@ -408,7 +408,7 @@ void InitTLS() // This is strictly a one-time call!
 /*                                 F a t a l                                  */
 /******************************************************************************/
   
-void Fatal(std::string *eMsg, const char *msg)
+void Fatal(std::string *eMsg, const char *msg, bool sslmsg=false)
 {
 // If there is an outboard error string object, return the message there.
 //
@@ -417,7 +417,7 @@ void Fatal(std::string *eMsg, const char *msg)
 // Now route the message to the message callback function. We always flush the
 // ssl error queue, even if not an ssl error, to prevent suprises.
 //
-   XrdTlsGlobal::msgCB("TLS_Context:", msg, true);
+   XrdTlsGlobal::msgCB("TLS_Context:", msg, sslmsg);
 }
 
 /******************************************************************************/
@@ -937,9 +937,9 @@ int XrdTlsContext::SessionCache(int opts, const char *id, int idlen)
 // Set the id is so wanted
 //
    if (id && idlen > 0)
-      {unsigned const char *idU = (unsigned const char *)id;
-       unsigned int         idL = (unsigned int)idlen;
-       if (!SSL_CTX_set_session_id_context(pImpl->ctx,idU,idL)) opts |= scIdErr;
+      {if (!SSL_CTX_set_session_id_context(pImpl->ctx,
+                                          (unsigned const char *)id,
+                                          (unsigned int)idlen)) opts |= scIdErr;
       }
 
 // If a flush interval was specified and it is different from what we have
