@@ -42,8 +42,6 @@
 #include <sys/isa_defs.h>
 #endif
 
-#include <limits>
-
 #include "XrdVersion.hh"
 
 #include "XProtocol/XProtocol.hh"
@@ -58,7 +56,6 @@
 #include "XrdOuc/XrdOucReqID.hh"
 #include "XrdOuc/XrdOucString.hh"
 #include "XrdOuc/XrdOucStream.hh"
-#include "XrdOuc/XrdOucTrace.hh"
 #include "XrdOuc/XrdOucUtils.hh"
 #include "XrdSec/XrdSecLoadSecurity.hh"
 #include "XrdSys/XrdSysError.hh"
@@ -68,14 +65,11 @@
 #include "XrdTls/XrdTlsContext.hh"
 
 #include "XrdXrootd/XrdXrootdAdmin.hh"
-#include "XrdXrootd/XrdXrootdAio.hh"
 #include "XrdXrootd/XrdXrootdCallBack.hh"
 #include "XrdXrootd/XrdXrootdFile.hh"
 #include "XrdXrootd/XrdXrootdFileLock.hh"
 #include "XrdXrootd/XrdXrootdFileLock1.hh"
-#include "XrdXrootd/XrdXrootdGSReal.hh"
 #include "XrdXrootd/XrdXrootdJob.hh"
-#include "XrdXrootd/XrdXrootdMonitor.hh"
 #include "XrdXrootd/XrdXrootdPrepare.hh"
 #include "XrdXrootd/XrdXrootdProtocol.hh"
 #include "XrdXrootd/XrdXrootdStats.hh"
@@ -116,7 +110,7 @@ Where:
 
                 XrdOucString      *XrdXrootdCF;
 
-extern          XrdOucTrace       *XrdXrootdTrace;
+extern          XrdSysTrace        XrdXrootdTrace;
 
                 XrdXrootdPrepare  *XrdXrootdPrepQ;
 
@@ -134,6 +128,17 @@ extern XrdSfsFileSystem *XrdSfsGetDefaultFileSystem
                           const char       *configFn,
                           XrdOucEnv        *EnvInfo);
 
+/******************************************************************************/
+/*                        G l o b a l   S t a t i c s                         */
+/******************************************************************************/
+
+namespace XrdXrootd
+{
+extern XrdBuffManager       *BPool;
+extern XrdScheduler         *Sched;
+extern XrdXrootdStats       *SI;
+}
+  
 /******************************************************************************/
 /*                         L o c a l   S t a t i c s                          */
 /******************************************************************************/
@@ -147,6 +152,10 @@ char                    *gpfLib  = 0;// Normally zero for default
 char                    *gpfParm = 0;
 char                    *SecLib;
 int                      tlsCache= XrdTlsContext::scNone;
+int                      asyncFlags = 0;
+
+static const int asDebug   = 0x01;
+static const int asNoCache = 0x02;
 }
   
 /******************************************************************************/
@@ -177,10 +186,11 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 // Copy out the special info we want to use at top level
 //
    eDest.logger(pi->eDest->logger());
-   XrdXrootdTrace = new XrdOucTrace(&eDest);
+   XrdXrootdTrace.SetLogger(pi->eDest->logger());
    SI           = new XrdXrootdStats(pi->Stats);
-   Sched        = pi->Sched;
-   BPool        = pi->BPool;
+   XrdXrootd::SI= SI;
+   Sched        = pi->Sched; XrdXrootd::Sched = pi->Sched;
+   BPool        = pi->BPool; XrdXrootd::BPool = pi->BPool;
    hailWait     = pi->hailWait;
    readWait     = pi->readWait;
    Port         = pi->Port;
@@ -206,7 +216,6 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 // Pre-initialize some i/o values. Note that we now set maximum readv element
 // transfer size to the buffer size (before it was a reasonable 256K).
 //
-   if (!(as_miniosz = as_segsize/2)) as_miniosz = as_segsize;
    n = (pi->theEnv ? pi->theEnv->GetInt("MaxBuffSize") : 0);
    maxTransz = maxBuffsz = (n ? n : BPool->MaxSize());
    memset(Route, 0, sizeof(Route));
@@ -215,7 +224,7 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 //
    rdf = (parms && *parms ? parms : pi->ConfigFN);
    if (rdf && Config(rdf)) return 0;
-   if (pi->DebugON) XrdXrootdTrace->What = TRACE_ALL;
+   if (pi->DebugON) XrdXrootdTrace.What = TRACE_ALL;
 
 // Check if we are exporting a generic object name
 //
@@ -263,20 +272,16 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 //
    if (pi->theEnv) xrootdEnv.PutPtr("xrdEnv*", pi->theEnv);
 
-// Initialize monitoring (it won't do anything if it wasn't enabled)
+// Initialize monitoring (it won't do anything if it wasn't enabled). This
+// needs to be done before we load any plugins as plugins may need monitoring.
 //
-   if (!XrdXrootdMonitor::Init(Sched, &eDest, pi->myName, pi->myProg,
-                               myInst, Port)) return 0;
+   if (!ConfigMon(pi, xrootdEnv)) return 0;
 
-// Config g-stream objects, as needed. This needs to be done before we
-// load any plugins but after we initialize monitoring.
-//
-   ConfigGStream(xrootdEnv, pi->theEnv);
-
-// Get the filesystem to be used and its features
+// Get the filesystem to be used and its features.
 //
    if (!ConfigFS(xrootdEnv, pi->ConfigFN)) return 0;
    fsFeatures = osFS->Features();
+   isProxy = (fsFeatures & XrdSfs::hasPRXY) != 0;
    if (pi->theEnv) pi->theEnv->PutPtr("XrdSfsFileSystem*", osFS);
 
 // Check if the file system includes a custom prepare handler as this will
@@ -309,20 +314,30 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
           } while(tP);
       }
 
-// Initialiaze for AIO
+// Initialiaze for AIO. If we are not in debug mode and aio is enabled then we
+// turn off async I/O if tghe filesystem requests it or if this is a caching
+// proxy and we were asked not to use aio in such a cacse.
 //
-   if (!as_noaio) XrdXrootdAioReq::Init(as_segsize, as_maxperreq, as_maxpersrv);
-      else eDest.Say("Config asynchronous I/O has been disabled!");
+   if (!(asyncFlags & asDebug) && as_aioOK)
+      {if (fsFeatures & XrdSfs::hasNAIO) as_aioOK = 0;
+          else if (asyncFlags && asNoCache && fsFeatures & XrdSfs::hasCACH)
+                  as_aioOK = 0;
+       if (!as_aioOK) eDest.Say("Config asynchronous I/O has been disabled!");
+      }
+
+// Compute the maximum stutter allowed during async I/O (one per 64k)
+//
+   if (as_segsize > 65536) as_okstutter = as_segsize/65536;
 
 // Establish final sendfile processing mode. This may be turned off by the
-// link or by the SFS plugin usually because ot's a proxy.
+// link or by the SFS plugin usually because it's a proxy.
 //
    const char *why = 0;
    if (!as_nosf)
       {if (fsFeatures & XrdSfs::hasNOSF) why = "file system plugin.";
           else if (!XrdLink::sfOK) why = "OS kernel.";
        if (why)
-          {as_nosf = 1;
+          {as_nosf = true;
            eDest.Say("Config sendfile has been disabled by ", why);
           }
       }
@@ -330,11 +345,11 @@ int XrdXrootdProtocol::Configure(char *parms, XrdProtocol_Config *pi)
 // Create the file lock manager and initialize file handling
 //
    Locker = (XrdXrootdFileLock *)new XrdXrootdFileLock1();
-   XrdXrootdFile::Init(Locker, &eDest, as_nosf == 0);
+   XrdXrootdFile::Init(Locker, &eDest, !as_nosf);
 
 // Schedule protocol object cleanup (also advise the transit protocol)
 //
-   ProtStack.Set(pi->Sched, XrdXrootdTrace, TRACE_MEM);
+   ProtStack.Set(pi->Sched, &XrdXrootdTrace, TRACE_MEM);
    n = (pi->ConnMax/3 ? pi->ConnMax/3 : 30);
    ProtStack.Set(n, 60*60);
    XrdXrootdTransit::Init(pi->Sched, n, 60*60);
@@ -511,6 +526,7 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
              else if TS_Xeq("fsoverload",    xfso);
              else if TS_Xeq("gpflib",        xgpf);
              else if TS_Xeq("log",           xlog);
+             else if TS_Xeq("mongstream",    xmongs);
              else if TS_Xeq("monitor",       xmon);
              else if TS_Xeq("prep",          xprep);
              else if TS_Xeq("redirect",      xred);
@@ -542,8 +558,8 @@ int XrdXrootdProtocol::Config(const char *ConfigFN)
           {eDest.Say("Config failure: unable to setup TLS for protocol!");
            NoGo = 1;
           } else {
-           const char *sessID = "xroots";
-           tlsCtx->SessionCache(tlsCache, sessID, sizeof(sessID));
+           static const char *sessID = "xroots";
+           tlsCtx->SessionCache(tlsCache, sessID, 6);
           }
       }
 
@@ -663,37 +679,6 @@ bool XrdXrootdProtocol::ConfigFS(const char *path, XrdOucEnv &xEnv,
    return true;
 }
   
-/******************************************************************************/
-/*                         C o n f i g G S t r e a m                          */
-/******************************************************************************/
-
-void XrdXrootdProtocol::ConfigGStream(XrdOucEnv &myEnv, XrdOucEnv *urEnv)
-{
-   struct GSTable {const char *pin; int Mode; kXR_char Type; bool xrd;}
-   gsObj[] =      {{"ccm",    XROOTD_MON_CCM,   XROOTD_MON_GSCCM, false},
-                   {"pfc",    XROOTD_MON_PFC,   XROOTD_MON_GSPFC, false},
-                   {"TcpMon", XROOTD_MON_TCPMO, XROOTD_MON_GSTCP, true}
-                  };
-   int numgs = sizeof(gsObj)/sizeof(struct GSTable);
-   int flint = XrdXrootdMonitor::Flushing();
-   char vbuff[64];
-
-// For each enabled monitoring provider, allocate a g-stream and put
-// its address in our environment.
-//
-   for (int i = 0; i < numgs; i++)
-       {if (XrdXrootdMonitor::ModeEnabled(gsObj[i].Mode))
-           {XrdXrootdGStream *gs = new XrdXrootdGSReal(gsObj[i].pin,
-                                                       gsObj[i].Type,
-                                                       gsObj[i].Mode, flint);
-            snprintf(vbuff, sizeof(vbuff), "%s.gStream*", gsObj[i].pin);
-            if (!gsObj[i].xrd) myEnv.PutPtr(vbuff, (void *)gs);
-               else if (urEnv) urEnv->PutPtr(vbuff, (void *)gs);
-           }
-       }
-}
-
-  
 /******************************************************************************/
 /*                        C o n f i g S e c u r i t y                         */
 /******************************************************************************/
@@ -768,11 +753,13 @@ int XrdXrootdProtocol::ConfigSecurity(XrdOucEnv &xEnv, const char *cfn)
    Purpose:  To parse directive: async [limit <aiopl>] [maxsegs <msegs>]
                                        [maxtot <mtot>] [segsize <segsize>]
                                        [minsize <iosz>] [maxstalls <cnt>]
-                                       [force] [syncw] [off] [nosf]
+                                       [timeout <tos>]
+                                       [Debug] [force] [syncw] [off]
+                                       [nocache] [nosf]
 
-             <aiopl>  maximum number of async ops per link. Default 8.
+             <aiopl>  maximum number of async req per link. Default 8.
              <msegs>  maximum number of async ops per request. Default 8.
-             <mtot>   maximum number of async ops per server. Default is 20%
+             <mtot>   maximum number of async ops per server. Default is 4096.
                       of maximum connection times aiopl divided by two.
              <segsz>  The aio segment size. This is the maximum size that data
                       will be read or written. The defaults to 64K but is
@@ -780,12 +767,16 @@ int XrdXrootdProtocol::ConfigSecurity(XrdOucEnv &xEnv, const char *cfn)
              <iosz>   the minimum number of bytes that must be read or written
                       to allow async processing to occur (default is maxbsz/2
                       typically 1M).
+             <tos>    second timeout for async I/O.
              <cnt>    Maximum number of client stalls before synchronous i/o is
                       used. Async mode is tried after <cnt> requests.
+             Debug    Turns on async I/O for everything. This an internal
+                      undocumented option used for testing purposes.
              force    Uses async i/o for all requests, even when not explicitly
                       requested (this is compatible with synchronous clients).
              syncw    Use synchronous i/o for write requests.
              off      Disables async i/o
+             nocache  Disables async I/O is this is a caching proxy.
              nosf     Disables use of sendfile to send data to the client.
 
    Output: 0 upon success or 1 upon failure.
@@ -797,17 +788,20 @@ int XrdXrootdProtocol::xasync(XrdOucStream &Config)
     int  i, ppp;
     int  V_force=-1, V_syncw = -1, V_off = -1, V_mstall = -1, V_nosf = -1;
     int  V_limit=-1, V_msegs=-1, V_mtot=-1, V_minsz=-1, V_segsz=-1;
-    int  V_minsf=-1;
+    int  V_minsf=-1, V_debug=-1, V_noca=-1, V_tmo=-1;
     long long llp;
     struct asyncopts {const char *opname; int minv; int *oploc;
                       const char *opmsg;} asopts[] =
        {
+        {"Debug",     -1, &V_debug, ""},
         {"force",     -1, &V_force, ""},
         {"off",       -1, &V_off,   ""},
+        {"nocache",   -1, &V_noca,  ""},
         {"nosf",      -1, &V_nosf,  ""},
         {"syncw",     -1, &V_syncw, ""},
         {"limit",      0, &V_limit, "async limit"},
         {"segsize", 4096, &V_segsz, "async segsize"},
+        {"timeout",    0, &V_tmo,   "async timeout"},
         {"maxsegs",    0, &V_msegs, "async maxsegs"},
         {"maxstalls",  0, &V_mstall,"async maxstalls"},
         {"maxtot",     0, &V_mtot,  "async maxtot"},
@@ -862,18 +856,35 @@ int XrdXrootdProtocol::xasync(XrdOucStream &Config)
           }
       }
 
+// Calculate actual timeout
+//
+   if (V_tmo >= 0)
+      {i = V_tmo;
+       if (V_tmo < 1) i = 1;
+          else if (V_tmo > 360) i = 360;
+       if (i != V_tmo)
+          {char buff[64];
+           sprintf(buff, "%d readjusted to %d", V_tmo, i);
+           eDest.Emsg("Config", "async timeout", buff);
+           V_tmo = i;
+          }
+      }
+
 // Establish async options
 //
    if (V_limit > 0) as_maxperlnk = V_limit;
    if (V_msegs > 0) as_maxperreq = V_msegs;
    if (V_mtot  > 0) as_maxpersrv = V_mtot;
    if (V_minsz > 0) as_miniosz   = V_minsz;
-   if (V_segsz > 0) as_segsize   = V_segsz;
+   if (V_segsz > 0){as_segsize   = V_segsz; as_seghalf = V_segsz/2;}
+   if (V_tmo   > 0) as_timeout   = V_tmo;
    if (V_mstall> 0) as_maxstalls = V_mstall;
-   if (V_force > 0) as_force     = 1;
-   if (V_off   > 0) as_noaio     = 1;
-   if (V_syncw > 0) as_syncw     = 1;
-   if (V_nosf  > 0) as_nosf      = 1;
+   if (V_debug > 0) asyncFlags  |= asDebug;
+   if (V_force > 0) as_force     = true;
+   if (V_off   > 0) as_aioOK     = false;
+   if (V_syncw > 0) as_syncw     = true;
+   if (V_noca  > 0) asyncFlags  |= asNoCache;
+   if (V_nosf  > 0) as_nosf      = true;
    if (V_minsf > 0) as_minsfsz   = V_minsf;
 
    return 0;
@@ -1337,218 +1348,6 @@ int XrdXrootdProtocol::xlog(XrdOucStream &Config)
     return 0;
 }
 
-/******************************************************************************/
-/*                                  x m o n                                   */
-/******************************************************************************/
-
-/* Function: xmon
-
-   Purpose:  Parse directive: monitor [all] [auth]  [flush [io] <sec>]
-                                      [fstat <sec> [lfn] [ops] [ssq] [xfr <n>]
-                                      [fbuff <sz>]
-                                      [ident <sec>] [mbuff <sz>] [rbuff <sz>]
-                                      [rnums <cnt>] [window <sec>]
-                                      dest [Events] <host:port>
-
-   Events: [ccm] [files] [fstat] [info] [io] [iov] [pfc] [redir] [tcpmo] [user]
-
-         all                enables monitoring for all connections.
-         auth               add authentication information to "user".
-         flush  [io] <sec>  time (seconds, M, H) between auto flushes. When
-                            io is given applies only to i/o events.
-         fstat  <sec>       produces an "f" stream for open & close events
-                            <sec> specifies the flush interval (also see xfr)
-                            lfn    - adds lfn to the open event
-                            ops    - adds the ops record when the file is closed
-                            ssq    - computes the sum of squares for the ops rec
-                            xfr <n>- inserts i/o stats for open files every
-                                     <sec>*<n>. Minimum is 1.
-         fbsz   <sz>        size of message buffer for file stream monitoring.
-         ident  <sec>       time (seconds, M, H) between identification records.
-         mbuff  <sz>        size of message buffer for event trace monitoring.
-         rbuff  <sz>        size of message buffer for redirection monitoring.
-         rnums  <cnt>       bumber of redirections monitoring streams.
-         window <sec>       time (seconds, M, H) between timing marks.
-         dest               specified routing information. Up to two dests
-                            may be specified.
-         ccm                monitor cache context management
-         files              only monitors file open/close events.
-         fstats             vectors the "f" stream to the destination
-         info               monitors client appid and info requests.
-         io                 monitors I/O requests, and files open/close events.
-         iov                like I/O but also unwinds vector reads.
-         pfc                monitor proxy file cache
-         redir              monitors request redirections
-         user               monitors user login and disconnect events.
-         <host:port>        where monitor records are to be sentvia UDP.
-
-   Output: 0 upon success or !0 upon failure. Ignored by master.
-*/
-int XrdXrootdProtocol::xmon(XrdOucStream &Config)
-{   static const char *mrMsg[] = {"monitor mbuff value not specified",
-                                  "monitor rbuff value not specified",
-                                  "monitor mbuff", "monitor rbuff"
-                                 };
-    char  *val = 0, *cp, *monDest[2] = {0, 0};
-    long long tempval;
-    int i, monFlash = 0, monFlush=0, monMBval=0, monRBval=0, monWWval=0, monFbsz=0;
-    int    monIdent = 3600, xmode=0, monMode[2] = {0, 0}, mrType, *flushDest;
-    int    monRnums = 0, monFSint = 0, monFSopt = 0, monFSion = 0;
-    int    haveWord = 0;
-
-    while(haveWord || (val = Config.GetWord()))
-         {haveWord = 0;
-               if (!strcmp("all",  val)) xmode = XROOTD_MON_ALL;
-          else if (!strcmp("auth",  val))
-                  monMode[0] = monMode[1] = XROOTD_MON_AUTH;
-          else if (!strcmp("flush", val))
-                {if ((val = Config.GetWord()) && !strcmp("io", val))
-                    {    flushDest = &monFlash; val = Config.GetWord();}
-                    else flushDest = &monFlush;
-                 if (!val)
-                    {eDest.Emsg("Config", "monitor flush value not specified");
-                     return 1;
-                    }
-                 if (XrdOuca2x::a2tm(eDest,"monitor flush",val,
-                                           flushDest,1)) return 1;
-                }
-          else if (!strcmp("fstat",val))
-                  {if (!(val = Config.GetWord()))
-                      {eDest.Emsg("Config", "monitor fstat value not specified");
-                       return 1;
-                      }
-                   if (XrdOuca2x::a2tm(eDest,"monitor fstat",val,
-                                             &monFSint,0)) return 1;
-                   while((val = Config.GetWord()))
-                        if (!strcmp("lfn", val)) monFSopt |=  XROOTD_MON_FSLFN;
-                   else if (!strcmp("ops", val)) monFSopt |=  XROOTD_MON_FSOPS;
-                   else if (!strcmp("ssq", val)) monFSopt |=  XROOTD_MON_FSSSQ;
-                   else if (!strcmp("xfr", val))
-                           {if (!(val = Config.GetWord()))
-                               {eDest.Emsg("Config", "monitor fstat xfr count not specified");
-                                return 1;
-                               }
-                            if (XrdOuca2x::a2i(eDest,"monitor fstat io count",
-                                               val, &monFSion,1)) return 1;
-                            monFSopt |=  XROOTD_MON_FSXFR;
-                           }
-                   else {haveWord = 1; break;}
-                  }
-          else if (!strcmp("mbuff",val) || !strcmp("rbuff",val))
-                  {mrType = (*val == 'r');
-                   if (!(val = Config.GetWord()))
-                      {eDest.Emsg("Config",  mrMsg[mrType]); return 1;}
-                   if (XrdOuca2x::a2sz(eDest,mrMsg[mrType+2], val,
-                                             &tempval, 1024, 65536)) return 1;
-                   if (mrType) monRBval = static_cast<int>(tempval);
-                      else     monMBval = static_cast<int>(tempval);
-                  }
-         else if (!strcmp("fbsz", val))
-                {if (!(val = Config.GetWord()))
-                    {eDest.Emsg("Config", "monitor fbsz value not specified");
-                     return 1;
-                    }
-                 if (XrdOuca2x::a2sz(eDest,"monitor fbsz", val,
-                                             &tempval, 1024, 65472)) return 1;
-                  monFbsz = static_cast<int>(tempval);
-                }
-          else if (!strcmp("ident", val))
-                {if (!(val = Config.GetWord()))
-                    {eDest.Emsg("Config", "monitor ident value not specified");
-                     return 1;
-                    }
-                 if (XrdOuca2x::a2tm(eDest,"monitor ident",val,
-                                           &monIdent,0)) return 1;
-                }
-          else if (!strcmp("rnums", val))
-                {if (!(val = Config.GetWord()))
-                    {eDest.Emsg("Config", "monitor rnums value not specified");
-                     return 1;
-                    }
-                 if (XrdOuca2x::a2i(eDest,"monitor rnums",val, &monRnums,1,
-                                    XrdXrootdMonitor::rdrMax)) return 1;
-                }
-          else if (!strcmp("window", val))
-                {if (!(val = Config.GetWord()))
-                    {eDest.Emsg("Config", "monitor window value not specified");
-                     return 1;
-                    }
-                 if (XrdOuca2x::a2tm(eDest,"monitor window",val,
-                                           &monWWval,1)) return 1;
-                }
-          else break;
-         }
-
-    if (!val) {eDest.Emsg("Config", "monitor dest not specified"); return 1;}
-
-    for (i = 0; i < 2; i++)
-        {if (strcmp("dest", val)) break;
-         while((val = Config.GetWord()))
-                   if (!strcmp("ccm",  val)) monMode[i] |=  XROOTD_MON_CCM;
-              else if (!strcmp("files",val)) monMode[i] |=  XROOTD_MON_FILE;
-              else if (!strcmp("fstat",val)) monMode[i] |=  XROOTD_MON_FSTA;
-              else if (!strcmp("info", val)) monMode[i] |=  XROOTD_MON_INFO;
-              else if (!strcmp("io",   val)) monMode[i] |=  XROOTD_MON_IO;
-              else if (!strcmp("iov",  val)) monMode[i] |= (XROOTD_MON_IO
-                                                           |XROOTD_MON_IOV);
-              else if (!strcmp("pfc",  val)) monMode[i] |=  XROOTD_MON_PFC;
-              else if (!strcmp("redir",val)) monMode[i] |=  XROOTD_MON_REDR;
-              else if (!strcmp("tcpmon",val))monMode[i] |=  XROOTD_MON_TCPMO;
-              else if (!strcmp("user", val)) monMode[i] |=  XROOTD_MON_USER;
-              else break;
-          if (!val) {eDest.Emsg("Config","monitor dest value not specified");
-                     return 1;
-                    }
-          if (!(cp = index(val, (int)':')) || !atoi(cp+1))
-             {eDest.Emsg("Config","monitor dest port missing or invalid in",val);
-              return 1;
-             }
-          monDest[i] = strdup(val);
-         if (!(val = Config.GetWord())) break;
-        }
-
-    if (val)
-       {if (!strcmp("dest", val))
-           eDest.Emsg("Config", "Warning, a maximum of two dest values allowed.");
-           else eDest.Emsg("Config", "Warning, invalid monitor option", val);
-       }
-
-// Make sure dests differ
-//
-   if (monDest[0] && monDest[1] && !strcmp(monDest[0], monDest[1]))
-      {eDest.Emsg("Config", "Warning, monitor dests are identical.");
-       monMode[0] |= monMode[1]; monMode[1] = 0;
-       free(monDest[1]); monDest[1] = 0;
-      }
-
-// Add files option if I/O is enabled
-//
-   if (monMode[0] & XROOTD_MON_IO) monMode[0] |= XROOTD_MON_FILE;
-   if (monMode[1] & XROOTD_MON_IO) monMode[1] |= XROOTD_MON_FILE;
-
-// If ssq was specified, make sure we support IEEE754 floating point
-//
-#if !defined(__solaris__) || !defined(_IEEE_754)
-   if (monFSopt & XROOTD_MON_FSSSQ && !(std::numeric_limits<double>::is_iec559))
-      {monFSopt &= ~XROOTD_MON_FSSSQ;
-       eDest.Emsg("Config","Warning, 'fstat ssq' ignored; platform does not "
-                           "use IEEE754 floating point.");
-      }
-#endif
-
-// Set the monitor defaults
-//
-   XrdXrootdMonitor::Defaults(monMBval, monRBval, monWWval,
-                              monFlush, monFlash, monIdent, monRnums, monFbsz,
-                              monFSint, monFSopt, monFSion);
-
-   if (monDest[0]) monMode[0] |= (monMode[0] ? xmode : XROOTD_MON_FILE|xmode);
-   if (monDest[1]) monMode[1] |= (monMode[1] ? xmode : XROOTD_MON_FILE|xmode);
-   XrdXrootdMonitor::Defaults(monDest[0],monMode[0],monDest[1],monMode[1]);
-
-   return 0;
-}
-
 /******************************************************************************/
 /*                                 x p r e p                                  */
 /******************************************************************************/
@@ -2009,15 +1808,18 @@ int XrdXrootdProtocol::xtrace(XrdOucStream &Config)
        {
         {"all",      TRACE_ALL},
         {"auth",     TRACE_AUTH},
-        {"emsg",     TRACE_EMSG},
         {"debug",    TRACE_DEBUG},
+        {"emsg",     TRACE_EMSG},
         {"fs",       TRACE_FS},
+        {"fsaio",    TRACE_FSAIO},
+        {"fsio",     TRACE_FSIO},
         {"login",    TRACE_LOGIN},
         {"mem",      TRACE_MEM},
-        {"stall",    TRACE_STALL},
+        {"pgcserr",  TRACE_PGCS},
         {"redirect", TRACE_REDIR},
         {"request",  TRACE_REQ},
-        {"response", TRACE_RSP}
+        {"response", TRACE_RSP},
+        {"stall",    TRACE_STALL}
        };
     int i, neg, trval = 0, numopts = sizeof(tropts)/sizeof(struct traceopts);
 
@@ -2038,7 +1840,7 @@ int XrdXrootdProtocol::xtrace(XrdOucStream &Config)
                   }
           val = Config.GetWord();
          }
-    XrdXrootdTrace->What = trval;
+    XrdXrootdTrace.What = trval;
     return 0;
 }
 
