@@ -57,7 +57,7 @@
 /*                               G l o b a l s                                */
 /******************************************************************************/
 
-extern XrdOucTrace *XrdXrootdTrace;
+extern XrdSysTrace  XrdXrootdTrace;
 
 namespace
 {
